import {Component, ViewChild, Input, Output, ViewEncapsulation, EventEmitter, HostBinding, HostListener} from '@angular/core';
import template from './editor.html!text';

@Component({
  selector: 'ngc-editor',
  host: {
    class: 'editor'
  },
  template,
  encapsulation: ViewEncapsulation.None
})
export class Editor {
  // Zapamiętanie referencji do widoku potomnego w lokalnej zmiennej.
  @ViewChild('editableContentElement') editableContentElement;
  // Edytowana i wyświetlana treść.
  @Input() content;
  // Utworzenie dowiązania do atrybutu class elementu hostującego, który będzie zależał od właściwości editMode.
  @Input() @HostBinding('class.editor--edit-mode') editMode;
  @Input() showControls;
  @Output() editSaved = new EventEmitter();
  @Output() editableInput = new EventEmitter();

  // Musimy mieć pewność, aby odwzorować zmianę w edytowanym elemencie, jeśli treść zostanie zaktualizowana z zewnątrz.
  ngOnChanges() {
    if (this.editableContentElement && this.content) {
      this.setEditableContent(this.content);
    }
  }

  ngAfterViewInit() {
    this.setEditableContent(this.content);
  }

  // Zwraca zawartość edytowanej treści.
  getEditableContent() {
    return this.editableContentElement.nativeElement.textContent;
  }

  // Ustawia zawartość edytowanej treści.
  setEditableContent(content) {
    this.editableContentElement.nativeElement.textContent = content;
  }

  // Adnotacja utworzy kod nasłuchujący zdarzenia kliknięcia elementu hostującego, który spowoduje wywołanie kodu umieszczonego w funkcji.
  @HostListener('click')
  focusEditableContent() {
    if (this.editMode) {
      this.editableContentElement.nativeElement.focus();
    }
  }

  // Metoda wywoływana, jeśli edytowany element ulegnie zmianie.
  onInput() {
    // Wyemituj zdarzenie editableInput z edytowaną treścią.
    this.editableInput.next(this.getEditableContent());
  }

  // W przypadku zapisu pobieramy treść edytowanego elementu, przenosimy ją do pola content i emitujemy zdarzenie.
  save() {
    this.editSaved.next(this.getEditableContent());
    this.setEditableContent(this.content);
    // Ustawienie editMode na wartość false wyłącza tryb edycji.
    this.editMode = false;
  }

  // Anulowanie edycji nie kopiuje edytowanej zawartości i przełącza element w tryb podglądu.
  cancel() {
    this.setEditableContent(this.content);
    this.editableInput.next(this.getEditableContent());
    this.editMode = false;
  }

  // Metoda ta powoduje włączenie edytowalnego elementu i przejście w tryb edycji.
  edit() {
    this.editMode = true;
  }
}
